@@ -108,6 +108,16 @@ class BaseContext:
         """
         return self._portfolio.market_value
 
+    @property
+    def win_rate(self) -> Decimal:
+        """Running win rate of trades."""
+        return self._portfolio.win_rate
+
+    @property
+    def loss_rate(self) -> Decimal:
+        """Running loss rate of trades."""
+        return self._portfolio.loss_rate
+
     def orders(self) -> Iterator[Order]:
         r""":class:`Iterator` of all :class:`pybroker.portfolio.Order`\ s that
         have been placed and filled.
@@ -344,6 +354,7 @@ class ExecResult:
     buy_fill_price: Union[
         int,
         float,
+        np.floating,
         Decimal,
         PriceType,
         Callable[[str, BarData], Union[int, float, Decimal]],
@@ -351,6 +362,7 @@ class ExecResult:
     sell_fill_price: Union[
         int,
         float,
+        np.floating,
         Decimal,
         PriceType,
         Callable[[str, BarData], Union[int, float, Decimal]],
@@ -392,6 +404,9 @@ class PosSizeContext(BaseContext):
     r"""Holds data for a position size handler set with
     :meth:`pybroker.Strategy.set_pos_size_handler`. Used to set position sizes
     when placing orders from buy and sell signals.
+
+    Attributes:
+        sessions: ``dict`` used to store custom data for all symbols.
     """
 
     def __init__(
@@ -403,6 +418,7 @@ class PosSizeContext(BaseContext):
         pred_scope: PredictionScope,
         pending_order_scope: PendingOrderScope,
         models: Mapping[ModelSymbol, TrainedModel],
+        sessions: Mapping[str, Mapping],
         sym_end_index: Mapping[str, int],
         max_long_positions: Optional[int],
         max_short_positions: Optional[int],
@@ -417,6 +433,7 @@ class PosSizeContext(BaseContext):
             models=models,
             sym_end_index=sym_end_index,
         )
+        self.sessions = sessions
         self._signal_shares: dict[int, Union[int, float, Decimal]] = {}
         self._buy_results: Optional[list[ExecResult]] = None
         self._sell_results: Optional[list[ExecResult]] = None
@@ -565,6 +582,7 @@ class ExecContext(BaseContext):
 
     def __init__(
         self,
+        symbol: str,
         portfolio: Portfolio,
         col_scope: ColumnScope,
         ind_scope: IndicatorScope,
@@ -573,6 +591,7 @@ class ExecContext(BaseContext):
         pending_order_scope: PendingOrderScope,
         models: Mapping[ModelSymbol, TrainedModel],
         sym_end_index: Mapping[str, int],
+        session: Mapping,
     ):
         super().__init__(
             portfolio=portfolio,
@@ -585,15 +604,15 @@ class ExecContext(BaseContext):
             sym_end_index=sym_end_index,
         )
         self._scope = StaticScope.instance()
-        self._end_index: Optional[int] = None
         self._curr_date: Optional[np.datetime64] = None
         self._dt: Optional[datetime] = None
         self._foreign: dict[str, pd.DataFrame] = {}
 
-        self.symbol: Optional[str] = None
+        self.symbol: str = symbol
         self.buy_fill_price: Union[
             int,
             float,
+            np.floating,
             Decimal,
             PriceType,
             Callable[[str, BarData], Union[int, float, Decimal]],
@@ -603,6 +622,7 @@ class ExecContext(BaseContext):
         self.sell_fill_price: Union[
             int,
             float,
+            np.floating,
             Decimal,
             PriceType,
             Callable[[str, BarData], Union[int, float, Decimal]],
@@ -611,7 +631,7 @@ class ExecContext(BaseContext):
         self.sell_limit_price: Optional[Union[int, float, Decimal]] = None
         self.hold_bars: Optional[int] = None
         self.score: Optional[float] = None
-        self.session: Optional[dict] = None
+        self.session = session
 
         self.stop_loss: Optional[Union[int, float, Decimal]] = None
         self.stop_loss_pct: Optional[Union[int, float, Decimal]] = None
@@ -630,9 +650,7 @@ class ExecContext(BaseContext):
     @property
     def bars(self) -> int:
         """Number of bars of data that have completed."""
-        if not self._end_index:
-            return 0
-        return self._end_index
+        return self._sym_end_index[self.symbol]
 
     @property
     def dt(self) -> datetime:
@@ -650,7 +668,7 @@ class ExecContext(BaseContext):
         return self._col_scope.fetch(  # type: ignore[return-value]
             self.symbol,  # type: ignore[arg-type]
             DataCol.DATE.value,
-            self._end_index,
+            self._sym_end_index[self.symbol],
         )
 
     @property
@@ -660,7 +678,7 @@ class ExecContext(BaseContext):
         return self._col_scope.fetch(  # type: ignore[return-value]
             self.symbol,  # type: ignore[arg-type]
             DataCol.OPEN.value,
-            self._end_index,
+            self._sym_end_index[self.symbol],
         )
 
     @property
@@ -670,7 +688,7 @@ class ExecContext(BaseContext):
         return self._col_scope.fetch(  # type: ignore[return-value]
             self.symbol,  # type: ignore[arg-type]
             DataCol.HIGH.value,
-            self._end_index,
+            self._sym_end_index[self.symbol],
         )
 
     @property
@@ -680,7 +698,7 @@ class ExecContext(BaseContext):
         return self._col_scope.fetch(  # type: ignore[return-value]
             self.symbol,  # type: ignore[arg-type]
             DataCol.LOW.value,
-            self._end_index,
+            self._sym_end_index[self.symbol],
         )
 
     @property
@@ -690,7 +708,7 @@ class ExecContext(BaseContext):
         return self._col_scope.fetch(  # type: ignore[return-value]
             self.symbol,  # type: ignore[arg-type]
             DataCol.CLOSE.value,
-            self._end_index,
+            self._sym_end_index[self.symbol],
         )
 
     @property
@@ -700,7 +718,7 @@ class ExecContext(BaseContext):
         return self._col_scope.fetch(  # type: ignore[return-value]
             self.symbol,  # type: ignore[arg-type]
             DataCol.VOLUME.value,
-            self._end_index,
+            self._sym_end_index[self.symbol],
         )
 
     @property
@@ -710,7 +728,7 @@ class ExecContext(BaseContext):
         return self._col_scope.fetch(  # type: ignore[return-value]
             self.symbol,  # type: ignore[arg-type]
             DataCol.VWAP.value,
-            self._end_index,
+            self._sym_end_index[self.symbol],
         )
 
     def sell_all_shares(self):
@@ -891,10 +909,6 @@ class ExecContext(BaseContext):
         """
         self._pending_order_scope.remove_all(symbol)
 
-    def cancel_stop(self, stop_id: int) -> bool:
-        """Cancels a :class:`pybroker.portfolio.Stop` with ``stop_id``."""
-        return self._portfolio.remove_stop(stop_id)
-
     def cancel_stops(
         self,
         val: Union[str, Position, Entry],
@@ -927,6 +941,7 @@ class ExecContext(BaseContext):
             Union[
                 int,
                 float,
+                np.floating,
                 Decimal,
                 PriceType,
                 Callable[[str, BarData], Union[int, float, Decimal]],
@@ -944,7 +959,7 @@ class ExecContext(BaseContext):
                 points_dec = to_decimal(points)
         if limit_price is not None:
             limit_price_dec = to_decimal(limit_price)
-        self._stop_id += 1
+        ExecContext._stop_id += 1
         return Stop(
             id=self._stop_id,
             symbol=self._get_symbol(),
@@ -1134,25 +1149,20 @@ class ExecContext(BaseContext):
         if attr in self._scope.custom_data_cols:
             if self.symbol is None:
                 raise ValueError("symbol is not set.")
-            return self._col_scope.fetch(self.symbol, attr, self._end_index)
+            return self._col_scope.fetch(
+                self.symbol, attr, self._sym_end_index[self.symbol]
+            )
         raise AttributeError(f"Attribute {attr!r} not found.")
 
 
-def set_exec_ctx_data(
-    ctx: ExecContext, session: dict, symbol: str, date: np.datetime64
-):
+def set_exec_ctx_data(ctx: ExecContext, date: np.datetime64):
     """Sets data on an :class:`.ExecContext` instance.
 
     Args:
         ctx: :class:`.ExecContext`.
-        session: Custom session data.
-        symbol: Ticker symbol.
         date: Current bar's date.
     """
-    ctx.session = session
-    ctx.symbol = symbol
     ctx._curr_date = date
-    ctx._end_index = ctx._sym_end_index[symbol]
     ctx._dt = None
     ctx._foreign.clear()
     ctx.buy_fill_price = PriceType.MIDDLE
