@@ -29,7 +29,7 @@ from pybroker.context import (
     set_exec_ctx_data,
     set_pos_size_ctx_data,
 )
-from pybroker.portfolio import Order, Portfolio, Position, Stop, Trade
+from pybroker.portfolio import Order, Portfolio, Position, Trade
 
 
 @pytest.fixture()
@@ -123,6 +123,7 @@ def ctx(
     date,
 ):
     ctx = ExecContext(
+        symbol=symbol,
         portfolio=portfolio,
         col_scope=col_scope,
         ind_scope=ind_scope,
@@ -131,8 +132,9 @@ def ctx(
         pending_order_scope=pending_order_scope,
         models=trained_models,
         sym_end_index=sym_end_index,
+        session=session,
     )
-    set_exec_ctx_data(ctx, session, symbol, date)
+    set_exec_ctx_data(ctx, date)
     return ctx
 
 
@@ -158,6 +160,7 @@ def ctx_with_pos(
         sym: Position(sym, 100, "short") for sym in symbols
     }
     ctx = ExecContext(
+        symbol=symbol,
         portfolio=portfolio,
         col_scope=col_scope,
         ind_scope=ind_scope,
@@ -166,8 +169,9 @@ def ctx_with_pos(
         pending_order_scope=pending_order_scope,
         models=trained_models,
         sym_end_index=sym_end_index,
+        session=session,
     )
-    set_exec_ctx_data(ctx, session, symbol, date)
+    set_exec_ctx_data(ctx, date)
     return ctx
 
 
@@ -189,7 +193,10 @@ def ctx_with_orders(
 ):
     portfolio.orders = deque(orders)
     portfolio.trades = deque(trades)
+    portfolio.win_rate = 1
+    portfolio.lose_rate = 0
     ctx = ExecContext(
+        symbol=symbol,
         portfolio=portfolio,
         col_scope=col_scope,
         ind_scope=ind_scope,
@@ -198,8 +205,9 @@ def ctx_with_orders(
         pending_order_scope=pending_order_scope,
         models=trained_models,
         sym_end_index=sym_end_index,
+        session=session,
     )
-    set_exec_ctx_data(ctx, session, symbol, date)
+    set_exec_ctx_data(ctx, date)
     return ctx
 
 
@@ -207,6 +215,14 @@ def test_dt(ctx, date):
     assert ctx.dt == to_datetime(date)
 
 
+def test_win_rate(ctx_with_orders):
+    assert ctx_with_orders.win_rate == 1
+
+
+def test_loss_rate(ctx_with_orders):
+    assert ctx_with_orders.loss_rate == 0
+
+
 @pytest.mark.parametrize(
     "field", ["date", "open", "high", "low", "close", "volume", "adj_close"]
 )
@@ -456,34 +472,27 @@ def test_to_result(ctx, symbol, date):
     ctx.hold_bars = 2
     ctx.score = 7
     result = ctx.to_result()
-    assert result == ExecResult(
-        symbol=symbol,
-        date=date,
-        buy_fill_price=PriceType.AVERAGE,
-        buy_shares=20,
-        buy_limit_price=Decimal("99.99"),
-        sell_fill_price=PriceType.HIGH,
-        sell_shares=20,
-        sell_limit_price=Decimal("110.11"),
-        hold_bars=2,
-        score=7,
-        long_stops=frozenset(
-            [
-                Stop(
-                    id=1,
-                    symbol=symbol,
-                    stop_type=StopType.BAR,
-                    pos_type="long",
-                    percent=None,
-                    points=None,
-                    bars=2,
-                    fill_price=PriceType.HIGH,
-                    limit_price=None,
-                )
-            ]
-        ),
-        short_stops=None,
-    )
+    assert result.symbol == symbol
+    assert result.date == date
+    assert result.buy_fill_price == PriceType.AVERAGE
+    assert result.buy_shares == 20
+    assert result.buy_limit_price == Decimal("99.99")
+    assert result.sell_fill_price == PriceType.HIGH
+    assert result.sell_shares == 20
+    assert result.sell_limit_price == Decimal("110.11")
+    assert result.hold_bars == 2
+    assert result.score == 7
+    assert len(result.long_stops) == 1
+    assert result.short_stops is None
+    stop = next(iter(result.long_stops))
+    assert stop.symbol == symbol
+    assert stop.stop_type == StopType.BAR
+    assert stop.pos_type == "long"
+    assert stop.percent is None
+    assert stop.points is None
+    assert stop.bars == 2
+    assert stop.fill_price == PriceType.HIGH
+    assert stop.limit_price is None
 
 
 @pytest.mark.parametrize("pos_type", ["long", "short"])
@@ -509,50 +518,45 @@ def test_to_result_when_stop(
         percent = stop_amount
     else:
         points = stop_amount
-    expected_stops = frozenset(
-        [
-            Stop(
-                id=1,
-                symbol=symbol,
-                stop_type=expected_stop_type,
-                pos_type=pos_type,
-                percent=percent,
-                points=points,
-                bars=None,
-                fill_price=None,
-                limit_price=stop_limit,
-            )
-        ]
-    )
     buy_shares = None
     sell_shares = None
-    long_stops = None
-    short_stops = None
     if pos_type == "long":
         buy_shares = 100
-        long_stops = expected_stops
     else:
         sell_shares = 100
-        short_stops = expected_stops
     ctx.buy_shares = buy_shares
     ctx.sell_shares = sell_shares
     setattr(ctx, stop_attr, stop_amount)
     setattr(ctx, f"{stop_attr.replace('_pct', '')}_limit", stop_limit)
     result = ctx.to_result()
-    assert result == ExecResult(
-        symbol=symbol,
-        date=date,
-        buy_fill_price=PriceType.MIDDLE,
-        buy_shares=buy_shares,
-        buy_limit_price=None,
-        sell_fill_price=PriceType.MIDDLE,
-        sell_shares=sell_shares,
-        sell_limit_price=None,
-        hold_bars=None,
-        score=None,
-        long_stops=long_stops,
-        short_stops=short_stops,
-    )
+    assert result.symbol == symbol
+    assert result.date == date
+    assert result.buy_fill_price == PriceType.MIDDLE
+    assert result.buy_limit_price is None
+    assert result.sell_fill_price == PriceType.MIDDLE
+    assert result.sell_limit_price is None
+    assert result.hold_bars is None
+    assert result.score is None
+    if pos_type == "long":
+        assert result.buy_shares == 100
+        assert result.sell_shares is None
+        assert len(result.long_stops) == 1
+        assert result.short_stops is None
+        stop = next(iter(result.long_stops))
+    else:
+        assert result.sell_shares == 100
+        assert result.buy_shares is None
+        assert len(result.short_stops) == 1
+        assert result.long_stops is None
+        stop = next(iter(result.short_stops))
+    assert stop.symbol == symbol
+    assert stop.stop_type == expected_stop_type
+    assert stop.pos_type == pos_type
+    assert stop.percent == percent
+    assert stop.points == points
+    assert stop.bars is None
+    assert stop.fill_price is None
+    assert stop.limit_price == stop_limit
 
 
 @pytest.mark.parametrize(
@@ -588,10 +592,7 @@ def test_trades_when_empty(ctx):
 
 
 def test_set_exec_ctx_data(ctx, symbols, sym_end_index):
-    sym = symbols[-1]
-    session = {"a": 1, "b": 2}
     date = np.datetime64("2020-01-01")
-    ctx._end_index = 1000
     ctx._foreign = {"SPY": np.random.rand(100)}
     ctx.buy_fill_price = PriceType.AVERAGE
     ctx.buy_shares = 100
@@ -610,13 +611,10 @@ def test_set_exec_ctx_data(ctx, symbols, sym_end_index):
     ctx.stop_trailing = 100
     ctx.stop_trailing_pct = 15
     ctx.stop_trailing_limit = 80.8
-    set_exec_ctx_data(ctx, session, sym, date)
-    assert ctx.session == session
-    assert ctx.symbol == sym
+    set_exec_ctx_data(ctx, date)
     assert ctx.dt == to_datetime(date)
-    assert ctx.bars == sym_end_index[sym]
+    assert ctx.bars == sym_end_index[ctx.symbol]
     assert not ctx._foreign
-    assert ctx._end_index == sym_end_index[sym]
     assert ctx.buy_fill_price == PriceType.MIDDLE
     assert ctx.buy_shares is None
     assert ctx.buy_limit_price is None
@@ -693,6 +691,7 @@ def test_set_pos_ctx_data(
             short_stops=None,
         ),
     ]
+    sessions = {"SPY": {}, "AAPL": {}, "TSLA": {"foo": 1}}
     ctx = PosSizeContext(
         portfolio,
         col_scope,
@@ -701,11 +700,13 @@ def test_set_pos_ctx_data(
         pred_scope,
         pending_order_scope,
         trained_models,
+        sessions,
         sym_end_index,
         max_long_positions=1,
         max_short_positions=None,
     )
     set_pos_size_ctx_data(ctx, buy_results, sell_results)
+    assert ctx.sessions == sessions
     buy_signals = list(ctx.signals("buy"))
     assert len(buy_signals) == 1
     assert buy_signals[0].id == 0
