@@ -415,6 +415,13 @@ def test_r_squared(values, expected_r2):
 
 
 class TestEvaluateMixin:
+    @pytest.mark.parametrize(
+        "sharpe_length, expected_sharpe",
+        [
+            (None, 0.01710828175162464),
+            (252, 0.01710828175162464 * np.sqrt(252)),
+        ],
+    )
     @pytest.mark.parametrize(
         "bootstrap_sample_size, bootstrap_samples", [(10, 100), (100_000, 100)]
     )
@@ -425,6 +432,8 @@ class TestEvaluateMixin:
         portfolio_df,
         trades_df,
         calc_bootstrap,
+        sharpe_length,
+        expected_sharpe,
     ):
         mixin = EvaluateMixin()
         result = mixin.evaluate(
@@ -433,24 +442,30 @@ class TestEvaluateMixin:
             calc_bootstrap,
             bootstrap_sample_size=bootstrap_sample_size,
             bootstrap_samples=bootstrap_samples,
+            sharpe_length=sharpe_length,
         )
         assert result.metrics is not None
         if not calc_bootstrap:
             assert result.bootstrap is None
-            return
-        assert result.bootstrap is not None
-        ci = result.bootstrap.conf_intervals
-        assert ci.columns.tolist() == ["lower", "upper"]
-        names = ci.index.get_level_values(0).unique().tolist()
-        assert names == ["Log Profit Factor", "Sharpe Ratio"]
-        for name in names:
-            df = ci[ci.index.get_level_values(0) == name]
-            confs = df.index.get_level_values(1).tolist()
-            assert confs == ["97.5%", "95%", "90%"]
-        dd = result.bootstrap.drawdown_conf
-        assert dd.columns.tolist() == ["amount", "percent"]
-        conf = dd.index.get_level_values(0).tolist()
-        assert conf == ["99.9%", "99%", "95%", "90%"]
+        else:
+            assert result.bootstrap is not None
+            assert result.bootstrap.conf_intervals is not None
+            assert result.bootstrap.drawdown_conf is not None
+            assert result.bootstrap.profit_factor is not None
+            assert result.bootstrap.sharpe is not None
+            assert result.bootstrap.drawdown is not None
+            ci = result.bootstrap.conf_intervals
+            assert ci.columns.tolist() == ["lower", "upper"]
+            names = ci.index.get_level_values(0).unique().tolist()
+            assert names == ["Profit Factor", "Sharpe Ratio"]
+            for name in names:
+                df = ci[ci.index.get_level_values(0) == name]
+                confs = df.index.get_level_values(1).tolist()
+                assert confs == ["97.5%", "95%", "90%"]
+            dd = result.bootstrap.drawdown_conf
+            assert dd.columns.tolist() == ["amount", "percent"]
+            conf = dd.index.get_level_values(0).tolist()
+            assert conf == ["99.9%", "99%", "95%", "90%"]
         metrics = result.metrics
         assert metrics.initial_market_value == 500000
         assert metrics.end_market_value == 693111.87
@@ -480,12 +495,14 @@ class TestEvaluateMixin:
         assert metrics.avg_loss_pct == -2.9235326086956523
         assert metrics.avg_losing_trade_bars == 2.358695652173913
         assert metrics.largest_win == 21069.63
+        assert metrics.largest_win_pct == 14.49
         assert metrics.largest_win_bars == 3
         assert metrics.largest_loss == -11487.43
+        assert metrics.largest_loss_pct == -6.49
         assert metrics.largest_loss_bars == 3
         assert metrics.max_wins == 7
         assert metrics.max_losses == 7
-        assert metrics.sharpe == 0.01710828175162464
+        assert metrics.sharpe == expected_sharpe
         assert metrics.profit_factor == 1.0759385033768167
         assert metrics.ulcer_index == 1.898347959437099
         assert metrics.upi == 0.01844528848501509
@@ -501,6 +518,7 @@ class TestEvaluateMixin:
             calc_bootstrap,
             bootstrap_sample_size=10,
             bootstrap_samples=100,
+            sharpe_length=None,
         )
         assert result.metrics is not None
         for field in get_type_hints(EvalMetrics):
@@ -517,6 +535,7 @@ class TestEvaluateMixin:
             calc_bootstrap,
             bootstrap_sample_size=10,
             bootstrap_samples=100,
+            sharpe_length=None,
         )
         assert result.metrics is not None
         for field in get_type_hints(EvalMetrics):
@@ -531,23 +550,42 @@ class TestEvaluateMixin:
             calc_bootstrap,
             bootstrap_sample_size=10,
             bootstrap_samples=100,
+            sharpe_length=None,
         )
         metrics = result.metrics
         assert metrics is not None
-        assert metrics.trade_count == 0
+        assert metrics.total_pnl == 0
+        assert metrics.total_return_pct == 0
         assert metrics.total_profit == 0
         assert metrics.total_loss == 0
         assert metrics.win_rate == 0
         assert metrics.loss_rate == 0
+        assert metrics.winning_trades == 0
+        assert metrics.losing_trades == 0
+        assert metrics.avg_pnl == 0
+        assert metrics.avg_return_pct == 0
+        assert metrics.avg_trade_bars == 0
         assert metrics.avg_profit == 0
         assert metrics.avg_profit_pct == 0
+        assert metrics.avg_winning_trade_bars == 0
         assert metrics.avg_loss == 0
         assert metrics.avg_loss_pct == 0
+        assert metrics.avg_losing_trade_bars == 0
         assert metrics.largest_win == 0
+        assert metrics.largest_win_pct == 0
+        assert metrics.largest_win_bars == 0
         assert metrics.largest_loss == 0
+        assert metrics.largest_loss_pct == 0
+        assert metrics.largest_loss_bars == 0
         assert metrics.max_wins == 0
         assert metrics.max_losses == 0
+        assert metrics.total_fees == 0
         if calc_bootstrap:
             assert result.bootstrap is not None
+            assert result.bootstrap.conf_intervals is not None
+            assert result.bootstrap.drawdown_conf is not None
+            assert result.bootstrap.profit_factor is not None
+            assert result.bootstrap.sharpe is not None
+            assert result.bootstrap.drawdown is not None
         else:
             assert result.bootstrap is None
