@@ -22,11 +22,12 @@ import pandas as pd
 import pytest
 import re
 from .fixtures import *
-from collections import deque
+from collections import defaultdict, deque
 from datetime import datetime
 from decimal import Decimal
 from pybroker.common import DataCol, PriceType, to_datetime
 from pybroker.config import StrategyConfig
+from pybroker.context import ExecContext
 from pybroker.data import DataSource
 from pybroker.portfolio import (
     Order,
@@ -39,7 +40,6 @@ from pybroker.scope import PendingOrder
 from pybroker.strategy import (
     BacktestMixin,
     Execution,
-    ExecSymbol,
     Strategy,
     TestResult,
     WalkforwardMixin,
@@ -154,6 +154,8 @@ def pos_size_handler(ctx):
     signals = tuple(ctx.signals())
     ctx.set_shares(signals[0], shares=1000)
     ctx.set_shares(signals[1], shares=2000)
+    assert type(ctx.sessions["SPY"]) == dict
+    assert type(ctx.sessions["AAPL"]) == dict
 
 
 class TestBacktestMixin:
@@ -197,7 +199,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -252,7 +256,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -295,7 +301,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -341,7 +349,9 @@ class TestBacktestMixin:
         ):
             mixin.backtest_executions(
                 executions=execs,
-                sessions=self._sessions(execs),
+                before_exec_fn=None,
+                after_exec_fn=None,
+                sessions=defaultdict(dict),
                 models={},
                 indicator_data={},
                 test_data=data_source_df,
@@ -375,7 +385,9 @@ class TestBacktestMixin:
         ):
             mixin.backtest_executions(
                 executions=execs,
-                sessions=self._sessions(execs),
+                before_exec_fn=None,
+                after_exec_fn=None,
+                sessions=defaultdict(dict),
                 models={},
                 indicator_data={},
                 test_data=data_source_df,
@@ -401,7 +413,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -434,7 +448,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df[data_source_df["symbol"] != "AAPL"],
@@ -469,7 +485,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -505,7 +523,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -543,7 +563,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -594,7 +616,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -670,7 +694,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=df,
@@ -711,7 +737,9 @@ class TestBacktestMixin:
         with pytest.raises(ValueError, match=r"Unknown price: .*"):
             mixin.backtest_executions(
                 executions=execs,
-                sessions=self._sessions(execs),
+                before_exec_fn=None,
+                after_exec_fn=None,
+                sessions=defaultdict(dict),
                 models={},
                 indicator_data={},
                 test_data=data_source_df,
@@ -747,7 +775,9 @@ class TestBacktestMixin:
         ):
             mixin.backtest_executions(
                 executions=execs,
-                sessions=self._sessions(execs),
+                before_exec_fn=None,
+                after_exec_fn=None,
+                sessions=defaultdict(dict),
                 models={},
                 indicator_data={},
                 test_data=data_source_df,
@@ -783,7 +813,9 @@ class TestBacktestMixin:
         ):
             mixin.backtest_executions(
                 executions=execs,
-                sessions=self._sessions(execs),
+                before_exec_fn=None,
+                after_exec_fn=None,
+                sessions=defaultdict(dict),
                 models={},
                 indicator_data={},
                 test_data=data_source_df,
@@ -815,7 +847,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -849,7 +883,9 @@ class TestBacktestMixin:
         mixin = BacktestMixin()
         mixin.backtest_executions(
             executions=execs,
-            sessions=self._sessions(execs),
+            before_exec_fn=None,
+            after_exec_fn=None,
+            sessions=defaultdict(dict),
             models={},
             indicator_data={},
             test_data=data_source_df,
@@ -863,13 +899,6 @@ class TestBacktestMixin:
         )
         assert not len(portfolio.orders)
 
-    def _sessions(self, execs):
-        return {
-            ExecSymbol(exec.id, sym): {}
-            for exec in execs
-            for sym in exec.symbols
-        }
-
 
 @pytest.fixture()
 def executions_train_only():
@@ -1791,13 +1820,13 @@ class TestStrategy:
                 ctx.buy_shares = 100
                 ctx.stop_loss = 10
 
-        df = data_source_df[data_source_df["symbol"] == "SPY"]
+        df = data_source_df[data_source_df["symbol"].isin(["SPY", "AAPL"])]
         dates = df["date"].unique()
         dates = dates[dates <= np.datetime64(END_DATE)]
         strategy = Strategy(data_source_df, START_DATE, END_DATE)
-        strategy.add_execution(exec_fn, "SPY")
+        strategy.add_execution(exec_fn, ["SPY", "AAPL"])
         result = strategy.backtest(calc_bootstrap=False)
-        assert len(result.trades) == 1
+        assert len(result.trades) == 2
         trade = result.trades.iloc[0]
         assert trade["type"] == "long"
         assert trade["symbol"] == "SPY"
@@ -1808,7 +1837,17 @@ class TestStrategy:
         assert trade["agg_pnl"] == -1000
         assert trade["pnl_per_bar"] == round(-1000 / trade["bars"], 2)
         assert trade["stop"] == "loss"
-        assert len(result.orders) == 2
+        trade = result.trades.iloc[1]
+        assert trade["type"] == "long"
+        assert trade["symbol"] == "AAPL"
+        assert trade["entry_date"] == dates[1]
+        assert trade["exit"] == trade["entry"] - 10
+        assert trade["shares"] == 100
+        assert trade["pnl"] == -1000
+        assert trade["agg_pnl"] == -2000
+        assert trade["pnl_per_bar"] == round(-1000 / trade["bars"], 2)
+        assert trade["stop"] == "loss"
+        assert len(result.orders) == 4
         buy_order = result.orders.iloc[0]
         assert buy_order["type"] == "buy"
         assert buy_order["symbol"] == "SPY"
@@ -1816,12 +1855,25 @@ class TestStrategy:
         assert buy_order["shares"] == 100
         assert np.isnan(buy_order["limit_price"])
         assert buy_order["fees"] == 0
-        sell_order = result.orders.iloc[1]
+        buy_order = result.orders.iloc[1]
+        assert buy_order["type"] == "buy"
+        assert buy_order["symbol"] == "AAPL"
+        assert buy_order["date"] == dates[1]
+        assert buy_order["shares"] == 100
+        assert np.isnan(buy_order["limit_price"])
+        assert buy_order["fees"] == 0
+        sell_order = result.orders.iloc[2]
         assert sell_order["type"] == "sell"
         assert sell_order["symbol"] == "SPY"
         assert sell_order["shares"] == 100
         assert np.isnan(sell_order["limit_price"])
         assert sell_order["fees"] == 0
+        sell_order = result.orders.iloc[3]
+        assert sell_order["type"] == "sell"
+        assert sell_order["symbol"] == "AAPL"
+        assert sell_order["shares"] == 100
+        assert np.isnan(sell_order["limit_price"])
+        assert sell_order["fees"] == 0
 
     def test_backtest_when_sell_before_stop_loss(self, data_source_df):
         def exec_fn(ctx):
@@ -1861,30 +1913,6 @@ class TestStrategy:
         assert np.isnan(sell_order["limit_price"])
         assert sell_order["fees"] == 0
 
-    def test_backtest_when_cancel_stop(self, data_source_df):
-        def exec_fn(ctx):
-            if ctx.bars == 1:
-                ctx.buy_shares = 100
-                ctx.stop_loss = 10
-            elif ctx.bars == 10:
-                assert ctx.cancel_stop(stop_id=1)
-
-        df = data_source_df[data_source_df["symbol"] == "SPY"]
-        dates = df["date"].unique()
-        dates = dates[dates <= np.datetime64(END_DATE)]
-        strategy = Strategy(data_source_df, START_DATE, END_DATE)
-        strategy.add_execution(exec_fn, "SPY")
-        result = strategy.backtest(calc_bootstrap=False)
-        assert not len(result.trades)
-        assert len(result.orders) == 1
-        buy_order = result.orders.iloc[0]
-        assert buy_order["type"] == "buy"
-        assert buy_order["symbol"] == "SPY"
-        assert buy_order["date"] == dates[1]
-        assert buy_order["shares"] == 100
-        assert np.isnan(buy_order["limit_price"])
-        assert buy_order["fees"] == 0
-
     def test_backtest_when_cancel_stops(self, data_source_df):
         def exec_fn(ctx):
             if ctx.bars == 1:
@@ -1941,3 +1969,108 @@ class TestStrategy:
         result = strategy.backtest(calc_bootstrap=False)
         assert len(result.trades) == 1
         assert result.trades.iloc[0]["stop"] is None
+
+    def test_backtest_when_before_exec(self, data_source_df):
+        def before_exec_fn(ctxs):
+            assert len(ctxs) == 2
+            assert type(ctxs["SPY"]) == ExecContext
+            assert type(ctxs["AAPL"]) == ExecContext
+            ctxs["SPY"].session["foo"] = "bar"
+
+        def exec_fn(ctx):
+            if ctx.symbol == "AAPL" and not ctx.long_pos():
+                ctx.buy_shares = 200
+            if ctx.symbol == "SPY":
+                assert ctx.session["foo"] == "bar"
+
+        df = data_source_df[data_source_df["symbol"] == "AAPL"]
+        dates = df["date"].unique()
+        dates = dates[dates <= np.datetime64(END_DATE)]
+        strategy = Strategy(data_source_df, START_DATE, END_DATE)
+        strategy.add_execution(exec_fn, ["SPY", "AAPL"])
+        strategy.set_before_exec(before_exec_fn)
+        result = strategy.backtest(calc_bootstrap=False)
+        assert len(result.orders) == 1
+        buy_order = result.orders.iloc[0]
+        assert buy_order["type"] == "buy"
+        assert buy_order["symbol"] == "AAPL"
+        assert buy_order["date"] == dates[1]
+        assert buy_order["shares"] == 200
+        assert np.isnan(buy_order["limit_price"])
+        assert buy_order["fees"] == 0
+
+    def test_backtest_when_before_exec_and_no_executions(self, data_source_df):
+        def before_exec_fn(ctxs):
+            assert len(ctxs) == 2
+            assert type(ctxs["SPY"]) == ExecContext
+            assert type(ctxs["AAPL"]) == ExecContext
+            if not ctxs["AAPL"].long_pos():
+                ctxs["AAPL"].buy_shares = 200
+
+        df = data_source_df[data_source_df["symbol"] == "AAPL"]
+        dates = df["date"].unique()
+        dates = dates[dates <= np.datetime64(END_DATE)]
+        strategy = Strategy(data_source_df, START_DATE, END_DATE)
+        strategy.add_execution(None, ["SPY", "AAPL"])
+        strategy.set_before_exec(before_exec_fn)
+        result = strategy.backtest(calc_bootstrap=False)
+        assert len(result.orders) == 1
+        buy_order = result.orders.iloc[0]
+        assert buy_order["type"] == "buy"
+        assert buy_order["symbol"] == "AAPL"
+        assert buy_order["date"] == dates[1]
+        assert buy_order["shares"] == 200
+        assert np.isnan(buy_order["limit_price"])
+        assert buy_order["fees"] == 0
+
+    def test_backtest_when_after_exec(self, data_source_df):
+        def after_exec_fn(ctxs):
+            assert len(ctxs) == 2
+            assert type(ctxs["SPY"]) == ExecContext
+            assert type(ctxs["AAPL"]) == ExecContext
+            if not ctxs["AAPL"].long_pos():
+                ctxs["AAPL"].buy_shares = 300
+
+        def exec_fn(ctx):
+            if ctx.symbol == "AAPL" and not ctx.long_pos():
+                ctx.buy_shares = 200
+
+        df = data_source_df[data_source_df["symbol"] == "AAPL"]
+        dates = df["date"].unique()
+        dates = dates[dates <= np.datetime64(END_DATE)]
+        strategy = Strategy(data_source_df, START_DATE, END_DATE)
+        strategy.add_execution(exec_fn, ["SPY", "AAPL"])
+        strategy.set_after_exec(after_exec_fn)
+        result = strategy.backtest(calc_bootstrap=False)
+        assert len(result.orders) == 1
+        buy_order = result.orders.iloc[0]
+        assert buy_order["type"] == "buy"
+        assert buy_order["symbol"] == "AAPL"
+        assert buy_order["date"] == dates[1]
+        assert buy_order["shares"] == 300
+        assert np.isnan(buy_order["limit_price"])
+        assert buy_order["fees"] == 0
+
+    def test_backtest_when_after_exec_and_no_executions(self, data_source_df):
+        def after_exec_fn(ctxs):
+            assert len(ctxs) == 2
+            assert type(ctxs["SPY"]) == ExecContext
+            assert type(ctxs["AAPL"]) == ExecContext
+            if not ctxs["AAPL"].long_pos():
+                ctxs["AAPL"].buy_shares = 200
+
+        df = data_source_df[data_source_df["symbol"] == "AAPL"]
+        dates = df["date"].unique()
+        dates = dates[dates <= np.datetime64(END_DATE)]
+        strategy = Strategy(data_source_df, START_DATE, END_DATE)
+        strategy.add_execution(None, ["SPY", "AAPL"])
+        strategy.set_after_exec(after_exec_fn)
+        result = strategy.backtest(calc_bootstrap=False)
+        assert len(result.orders) == 1
+        buy_order = result.orders.iloc[0]
+        assert buy_order["type"] == "buy"
+        assert buy_order["symbol"] == "AAPL"
+        assert buy_order["date"] == dates[1]
+        assert buy_order["shares"] == 200
+        assert np.isnan(buy_order["limit_price"])
+        assert buy_order["fees"] == 0
